# jahplayer - A widget library representation.

# Copyright (C) 2006 VM Inc.
# Released under the GPL.
# For more information, see http://www.jahshaka.org.

import utilities
import jahwidgets
import openassetlib
import os.path

import sys

#
# Provides a wrapper for an oal set and an associated list of media items
#

class collection:

	# Constructor for the collection
	def __init__( self ):
		self.filename = utilities.get_absolute_reel( "default.reel" )
		self.clear( )

	# Clear the collection
	def clear( self ):
		self.set = openassetlib.create_set( )
		self.playlist = {}
		self.thumbs = {}
		self.waveforms = {}
		self.sequences = {}
		self.active = ""

	# Append an asset as a file name or as an open media
	def append( self, file ):
		asset = openassetlib.create_asset( file )
		asset.set( "file", file )
		if file.find( "/sequence:" ) != -1:
			asset.set( "sequence", 1 )
		self.set.push_back( asset )
		self.playlist[ asset.guid( ) ] = None

	# Get the current active media item
	def get_active_media( self ):
		return self.get_media( guid = self.active )

	# Set the active media item
	def set_active_media( self, media ):
		if media is not None and media.asset is not None and media.reel is self:
			self.active = media.asset.guid( )
			return True
		return False

	def is_active_media( self, media ):
		return media is not None and media.asset is not None and media.asset.guid() == self.active

	# Get a media item by index or guid
	def get_media( self, index = None, guid = None ):
		if index is not None:
			asset = self.set.at( index )
			if asset is not None: 
				guid = asset.guid( )

		if guid is not None and guid != "" and self.playlist.has_key( guid ):
			media = self.playlist[ guid ]
			if media is None:
				asset = self.set.find( guid )
				if asset is not None:
					if asset.size( ) < 5:
						if openassetlib.convert_to_image( asset ) is None:
							if not openassetlib.convert_to_media( asset ):
								asset.set( "file", utilities.get_absolute_path( "jahplayer-logo.jpg" ) )
					if asset.find( "file" ) is not None:
						image = openassetlib.restore_image( asset, str( "jahplayer::thumb" ) )
						if image is not None:
							media = jahwidgets.create_image( image )
						else:
							media = jahwidgets.create_media( str( asset.find( "file" ).second ) )
						media.full_path = asset.find( "file" ).second
						media.reel = self
						self.playlist[ asset.guid( ) ] = media
						self.thumbs[ asset.guid( ) ] = image
						media.asset = asset
						media.border = 1

					# Assign the name - only if it hasn't been defined
					if str( asset.find( "name" ).second ) == "":
						file = str( asset.find( "file" ).second )
						path, name = os.path.split( file )
						if name.find( "sequence:" ) == 0:
							if name.find( "regexp=1" ) != -1:
								path, name = os.path.split( path )
							path, name = os.path.split( path )
						asset.set( "name", name )

			return media

		return None

	# Get an asset by index, guid or media object
	def get_asset( self, index = None, guid = None, media = None ):
		if media is not None:
			index = self.find( media = media )
		elif guid is not None:
			index = self.find( guid = guid )
		else:
			index = self.find( guid = self.active )

		if index is not None:
			return self.set.at( index )

		return None

	# Remove an asset by media, asset, guid or index
	def remove( self, media = None, asset = None, guid = None, index = None ):
		if guid is not None:
			asset = self.set.find( guid )
			self.set.remove( asset )
			del self.playlist[ guid ]
			return

		if index is None:
			index = find( media, asset, guid )
		if index >= 0 and index < self.size( ):
			asset = self.set.at( index )
			guid = asset.guid( )
			self.set.remove( asset )
			del self.playlist[ guid ]
		return

	# Find the index of a media item, guid or asset
	def find( self, media = None, asset = None, guid = None ):
		if media is not None:
			for key in self.playlist.keys( ):
				if self.playlist[ key ] is media:
					guid = key
					break
		elif asset is not None:
			guid = asset.guid( )

		if guid is not None:
			i = 0
			for m in self.set:
				if m.guid( ) == guid: break
				i = i + 1
			return i

		return self.size( )

	# Assign a thumb to the specified asset
	def set_thumb( self, thumb, media = None, asset = None, guid = None ):
		index = self.find( media, asset, guid )
		asset = self.set.at( index )
		self.thumbs[ asset.guid( ) ] = thumb
		openassetlib.store_image( asset, self.thumbs[ asset.guid( ) ], str( "jahplayer::thumb" ) )
		return

	# Retrieve a thumb for a specified asset
	def get_thumb( self, media = None, asset = None, guid = None ):
		index = self.find( media, asset, guid )
		asset = self.set.at( index )
		if self.thumbs.has_key( asset.guid( ) ):
			return self.thumbs[ asset.guid( ) ]
		return None

	def set_sequence( self, media ):
		if media.asset is not None:
			sequence = str( media.asset.find( "sequence" ).first ) != ""
			if sequence:
				self.sequences[ media.asset.guid( ) ] = media

	def get_sequence( self, asset ):
		if self.sequences.has_key( asset.guid( ) ):
			return self.sequences[ asset.guid( ) ]
		return None

	# Assign a waveform to the specified asset
	def set_waveform( self, waveform, media = None, asset = None, guid = None ):
		index = self.find( media, asset, guid )
		asset = self.set.at( index )
		self.waveforms[ asset.guid( ) ] = waveform
		return

	# Retrieve a waveform for a specified asset
	def get_waveform( self, media = None, asset = None, guid = None ):
		index = self.find( media, asset, guid )
		asset = self.set.at( index )
		if self.waveforms.has_key( asset.guid( ) ):
			return self.waveforms[ asset.guid( ) ]
		return None

	# Return the size of the set
	def size( self ):
		if self.set is None:
			return 0
		return self.set.size( )

	# Load a collection of assets from an xml file
	def load( self, db_plugin, xmlfilepath ):
		self.filename = xmlfilepath
		self.clear()
		if db_plugin is None:
			print 'ERROR: database plugin is not available'
			return
		loaded_set = openassetlib.load_set(db_plugin, xmlfilepath)
		if loaded_set is not None: self.set = loaded_set
		for asset in self.set:
			self.playlist[ str( asset.guid( ) ) ] = None

	# Save collection of assets to an xml file
	def save( self, db_plugin, xmlfilepath = None ):
		if xmlfilepath is None:
			xmlfilepath = self.filename
		if db_plugin is None:
			print 'ERROR: database plugin is not available'
			return
		if not openassetlib.save_set(db_plugin, xmlfilepath, self.set):
			print 'ERROR: Failed to save collection to %s' % xmlfilepath
