# jahplayer - A widget library representation.

# Copyright (C) 2006 VM Inc.
# Released under the GPL.
# For more information, see http://www.jahshaka.org.

import jahwidgets
import jahstate
import utilities
import openpluginlib
import assets
import utilities

class timed_update( jahwidgets.timer_delegate ):
	def __init__( self, handler ):
		jahwidgets.timer_delegate.__init__( self )
		self.handler = handler

	def timer( self ):
		self.handler.player.refresh( )
		return True

#
# Scrubbing observer - ensures video is paused during scrub operations
# and original speed is resumed on release
#

class scrubbing_observer( openpluginlib.observer ):
	def __init__( self, player, prop ):
		openpluginlib.observer.__init__( self )
		self.player = player
		self.prop = prop
		self.speed = player.speed
		self.timed_update = timed_update( self )
		self.timer = jahwidgets.timer( )

	def updated( self, subject ):
		if self.prop.valueAsBool( ):
			self.timer.delegate( self.timed_update )
			self.timer.start( 50, 0 )
			self.speed = self.player.speed
			self.player.speed = 0
		else:
			self.timer.stop( )
			self.player.refresh( )
			self.player.speed = self.speed

#
# Transport controls
#

class speed_observer( openpluginlib.observer ):
	def __init__( self, player, speed ):
		openpluginlib.observer.__init__( self )
		self.player = player
		self.speed = speed
		
	def updated( self, subject ):
		self.player.speed = not self.player.speed

#
# Frame position
#

class frame_observer( openpluginlib.observer ):
	def __init__( self, widget, prop, position, absolute ):
		openpluginlib.observer.__init__( self )
		self.widget = widget
		self.prop = prop
		self.position = position
		self.absolute = absolute
		
	def updated( self, subject ):
		if self.widget.state.player.speed != 0:
			self.widget.state.player.speed = 0
		if ( self.absolute ):
			self.prop.set( self.position )
		else:
			self.prop.set( self.position + self.prop.valueAsInt() )
		self.widget.state.player.refresh( )

#
# Special case for end position
#

class to_start_observer( openpluginlib.observer ):
	def __init__( self, state, frame_prop, length_prop ):
		openpluginlib.observer.__init__( self )
		self.state = state
		self.frame_prop = frame_prop
		self.length_prop = length_prop
		
	def updated( self, subject ):
		if self.state.player.active.asset is not None:
			self.frame_prop.set( self.length_prop.valueAsInt( ) )
			self.state.player.refresh( )

class to_end_observer( openpluginlib.observer ):
	def __init__( self, state, frame_prop, length_prop ):
		openpluginlib.observer.__init__( self )
		self.state = state
		self.frame_prop = frame_prop
		self.length_prop = length_prop
		
	def updated( self, subject ):
		if self.state.player.active.asset is not None:
			self.frame_prop.set( self.length_prop.valueAsInt( ) )
			self.state.player.refresh( )

class button_observer( openpluginlib.observer ):
	def __init__( self, method, mode ):
		openpluginlib.observer.__init__( self )
		self.method = method
		self.mode = mode

	def updated( self, subject ):
		self.method( self.mode )

#
# The control widget
#

class widget:
	def __init__( self, parent, state ):
		self.state = state
		self.handle = jahwidgets.create_ui_from_file( utilities.get_absolute_path( "control.ui" ), parent )
		self.scribbler = jahwidgets.scribbler( utilities.get_absolute_path( "jahplayer-font.png" ), 16, 16, 3 )

		self.state.register_media_listener( self )

		# Timeline control
		self.frame_prop = openpluginlib.property( openpluginlib.createKeyFromString( "frame" ) )
		self.scrubbing_prop = openpluginlib.property( openpluginlib.createKeyFromString( "scrubbing" ) )
		self.length_prop = openpluginlib.property( openpluginlib.createKeyFromString( "length" ) )
		jahwidgets.connect( "timeline:value", self.handle, self.frame_prop )
		jahwidgets.connect( "timeline:pressed", self.handle, self.scrubbing_prop )
		jahwidgets.connect( "timeline:maximum", self.handle, self.length_prop )

		# In/out point handling
		self.ignore_points = False
		self.in_point = utilities.create_property_and_attach( self.handle, "timeline:inValue", utilities.call_method_observer( self.in_changed ) )
		self.out_point = utilities.create_property_and_attach( self.handle, "timeline:outValue", utilities.call_method_observer( self.out_changed ) )
		self.in_prop = utilities.create_property_and_attach( self.handle, "pb_in:clicked", utilities.call_method_observer( self.set_in ) )
		self.out_prop = utilities.create_property_and_attach( self.handle, "pb_out:clicked", utilities.call_method_observer( self.set_out ) )

		self.scrubbing_prop.attach( scrubbing_observer( self.state.player, self.scrubbing_prop ) )
		jahwidgets.connect( "player:frame", self.state.player_handle, self.frame_prop )
		jahwidgets.connect( "player:length", self.state.player_handle, self.length_prop )

		self.state.position_prop = self.frame_prop

		jahwidgets.connect( "total_frames:value", self.handle, self.length_prop )
		jahwidgets.connect( "frame:value", self.handle, self.frame_prop )

		self.fps_prop = openpluginlib.property( openpluginlib.createKeyFromString( "length" ) )
		jahwidgets.connect( "frame:fps", self.handle, self.fps_prop )

		# Transport controls
		self.stop_prop = openpluginlib.property( openpluginlib.createKeyFromString( "stop" ) )
		self.stop_prop.setAlwaysNotify( True )
		jahwidgets.connect( "stop:clicked", self.handle, self.stop_prop )
		self.stop_prop.attach( speed_observer( self.state.player, 0 ) )

		self.ffwd_prop = openpluginlib.property( openpluginlib.createKeyFromString( "ffwd" ) )
		self.ffwd_prop.setAlwaysNotify( True )
		jahwidgets.connect( "ffwd:clicked", self.handle, self.ffwd_prop )
		self.ffwd_prop.attach( frame_observer( self, self.frame_prop, 12, False ) )

		self.rew_prop = openpluginlib.property( openpluginlib.createKeyFromString( "rew" ) )
		self.rew_prop.setAlwaysNotify( True )
		jahwidgets.connect( "rewind:clicked", self.handle, self.rew_prop )
		self.rew_prop.attach( frame_observer( self, self.frame_prop, -12, False ) )

		self.start_prop = openpluginlib.property( openpluginlib.createKeyFromString( "start" ) )
		self.start_prop.setAlwaysNotify( True )
		jahwidgets.connect( "start:clicked", self.handle, self.start_prop )
		self.start_prop.attach( to_start_observer( self.state, self.frame_prop, self.in_point ) )

		self.end_prop = openpluginlib.property( openpluginlib.createKeyFromString( "end" ) )
		self.end_prop.setAlwaysNotify( True )
		jahwidgets.connect( "end:clicked", self.handle, self.end_prop )
		self.end_prop.attach( to_end_observer( self.state, self.frame_prop, self.out_point ) )

		self.back_prop = utilities.create_property_and_attach( self.handle, "back:clicked", utilities.call_method_observer( self.back_button ) )
		self.step_prop = utilities.create_property_and_attach( self.handle, "step:clicked", utilities.call_method_observer( self.step_button ) )

		self.new_reel_prop = utilities.create_property_and_attach( self.handle, "new_reel:clicked", utilities.call_method_observer( self.new_reel ) )
		self.clear_reel_prop = utilities.create_property_and_attach( self.handle, "clear_reel:clicked", utilities.call_method_observer( self.clear_reel ) )
		self.clear_reel_prop = utilities.create_property_and_attach( self.handle, "reel_prev:clicked", utilities.call_method_observer( self.reel_prev ) )
		self.clear_reel_prop = utilities.create_property_and_attach( self.handle, "reel_next:clicked", utilities.call_method_observer( self.reel_next ) )
		self.delete_prop = utilities.create_property_and_attach( self.handle, "delete:clicked", utilities.call_method_observer( self.delete ) )
		self.thumb_prop = utilities.create_property_and_attach( self.handle, "thumb:clicked", utilities.call_method_observer( self.thumb ) )

	def back_button( self, property ):
		self.state.player.speed = 0
		self.state.position_prop.set( self.state.player.active.position( ) - 1 )
		self.state.player.refresh( )

	def step_button( self, property ):
		self.state.player.speed = 0
		self.state.position_prop.set( self.state.player.active.position( ) + 1 )
		self.state.player.refresh( )

	def delete( self, property ):
		active = self.state.get_active_media( )
		if active is not None and active.asset is not None:
			self.state.play_next( )
			self.state.remove_media( active )

	def thumb( self, property ):
		if self.state.player.active.asset is not None:
			thumb = self.state.player.active.reel.get_media( guid = self.state.player.active.asset.guid( ) )
			image = self.state.player.active.image

			if image is not None:
				width, height = utilities.bind_size_to( image.height( ) * self.state.player.active.ar( ), image.height( ), 100, 60 )
				thumb.image = self.scribbler.rescale( image, width, height )
				thumb.border = 1
				thumb.reel.set_thumb( thumb.image, thumb )
				self.state.notify_reel_listeners( )

	def new_reel( self, property ):
		self.state.new_reel( )

	def reel_prev( self, property ):
		self.state.reel_prev( )

	def reel_next( self, property ):
		self.state.reel_next( )

	def clear_reel( self, property ):
		self.state.clear_reel( )

	def notify_media_change( self, media ):
		if media.asset is not None and str( media.asset.find( "frames" ).first ) != "":
			size = int( str( media.asset.find( "frames" ).second ) ) 
			self.ignore_points = True
			self.length_prop.set( size - 1 )
			self.in_point.set( media.in_point )
			self.out_point.set( media.out_point - 1 )
			self.fps_prop.set( int( round( self.state.player.active.fps ) ) )
			self.ignore_points = False
		else:
			self.length_prop.set( 1 )

	def in_changed( self, subject ):
		if not self.ignore_points:
			media = self.state.player.active
			requested = self.in_point.valueAsInt( )
			media.in_point = requested
			if media.asset is not None:
				media.asset.set( "jahplayer::in", requested )
			self.frame_prop.set( requested )

	def out_changed( self, subject ):
		if not self.ignore_points:
			media = self.state.player.active
			requested = self.out_point.valueAsInt( )
			media.out_point = requested + 1
			if media.asset is not None:
				media.asset.set( "jahplayer::out", requested + 1 )
			self.frame_prop.set( requested )

	def set_in( self, subject ):
		self.in_point.set( self.frame_prop.valueAsInt( ) )

	def set_out( self, subject ):
		self.out_point.set( self.frame_prop.valueAsInt( ) )

