# jahplayer - A widget library representation.

# Copyright (C) 2006 VM Inc.
# Released under the GPL.
# For more information, see http://www.jahshaka.org.

import jahwidgets
import jahstate
import utilities
import openpluginlib

#
# The common asset viewer
#

class asset_view:
	def __init__( self, parent ):
		self.handle = jahwidgets.create_ui_from_file( utilities.get_absolute_path( "asset_view.ui" ), parent )

#
# The directory list widget
#

class dir_list:
	def __init__( self, parent ):
		self.handle = jahwidgets.create_ui_from_file( utilities.get_absolute_path( "dir_list.ui" ), parent )

#
# The reel list widget
#

class reel_list:
	def __init__( self, parent ):
		self.handle = jahwidgets.create_ui_from_file( utilities.get_absolute_path( "reel_list.ui" ), parent )

#
# Asset loading dialog
#

class load_dialog:
	def __init__( self, parent, state ):
		self.state = state

	def show( self, value ):
		extensions = openpluginlib.registered_filters( True )
		extensions = extensions + extensions.upper( )
		files = jahwidgets.get_open_filenames( extensions, "", "load media" )
		self.state.process_media( files )

#
# Reel manager dialog
#

class manager_dialog:
	def __init__( self, parent, state ):
		self.widget = jahwidgets.create_ui_from_file( utilities.get_absolute_path( "manager.ui" ), parent )
		self.state = state
		self.asset_view = asset_view( jahwidgets.find( "display", self.widget ) )
		self.reel_list = reel_list( jahwidgets.find( "source", self.widget ) )

	def show( self, value ):
		self.widget.show( value )


#
# The footer widget
#

class widget:
	def __init__( self, parent, state ):
		self.state = state
		self.handle = jahwidgets.create_ui_from_file( utilities.get_absolute_path( "footer.ui" ), parent )
		self.load_prop = utilities.create_property_and_attach( self.handle, "pb_load:clicked", utilities.launch_dialog_observer( load_dialog( parent, state ) ) )
		self.manager_prop = utilities.create_property_and_attach( self.handle, "pb_manager:clicked", utilities.launch_dialog_observer( manager_dialog( parent, state ) ) )
		self.zoom_up_prop = utilities.create_property_and_attach( self.handle, "zoom_up:clicked", utilities.call_method_observer( self.zoom_up ) )
		self.zoom_down_prop = utilities.create_property_and_attach( self.handle, "zoom_down:clicked", utilities.call_method_observer( self.zoom_down ) )
		self.quit_prop = utilities.create_property_and_attach( self.handle, "quit:clicked", utilities.call_method_observer( self.quit ) )
		self.histogram = utilities.create_property_and_attach( self.handle, "pb_histogram:clicked", utilities.call_method_observer( self.histogram ) )
		self.vectorscope = utilities.create_property_and_attach( self.handle, "pb_vector:clicked", utilities.call_method_observer( self.vectorscope ) )

		# hide vectorscope and reel manager for the moment
		jahwidgets.find( "pb_manager", self.handle ).show( False );
		jahwidgets.find( "pb_vector", self.handle ).show( False );

	def zoom_up( self, property ):
		self.state.change_zoom( 0.05 )

	def zoom_down( self, property ):
		self.state.change_zoom( -0.05 )

	def quit( self, property ):
		self.state.app.quit( )

	def histogram( self, property ):
		if self.state.player.histogram == False:
			self.state.player.histogram = True
		else:
			self.state.player.histogram = False
		self.state.player.refresh( )

	def vectorscope( self, property ):
		if self.state.player.histogram == False:
			self.state.player.histogram = True
		else:
			self.state.player.histogram = False
		self.state.player.refresh( )
