#!/usr/bin/env python

# jahplayer - A widget library representation.

# Copyright (C) 2006 VM Inc.
# Released under the GPL.
# For more information, see http://www.jahshaka.org.

#
# Mandatory for openpluginlib initialisation
#

import platform
import sys
import os
import os.path

# This is required for the executable generated from py2exe
# to use the preinstalled openlibraries python bindings

sys.path.append(os.environ['PYTHONPATH'])

# Initialise the dynamic linking functionality - note that the ubuntu amd64
# distro lacks the dl module, hence fallback to the non-symbolic values

if platform.system( ) == "Linux":
	try:
		import dl
		sys.setdlopenflags( dl.RTLD_NOW | dl.RTLD_GLOBAL )
	except:
		sys.setdlopenflags( 257 )

#
# Import jahplayer modules
#

import jahwidgets
import openpluginlib
import openassetlib
import uuid
import jahstate
import utilities
import assets

# Widgets

import header
import viewport
import control
import reel
import footer

#
# Define guid generator class acceptable to oal
#

class uuid_generator( openassetlib.guid_generator ):
	def generate_guid( self ):
		return uuid.uuid_time( )

#
# The jahplayer main application
#

class jahplayer:
	def __init__( self, state ):
		# Store the state
		self.state = state

		# Construct the main jahplayer window
		self.handle = jahwidgets.create_ui_from_file( utilities.get_absolute_path( "jahplayer.ui" ), jahwidgets.widget_handle() )

		# Populate the frames
		self.viewport = viewport.widget( jahwidgets.find( "viewport", self.handle ), state )
		self.header = header.widget( jahwidgets.find( "header", self.handle ), state )
		self.control = control.widget( jahwidgets.find( "control", self.handle ), state )
		self.reel = reel.widget( jahwidgets.find( "reel", self.handle ), state )
		self.footer = footer.widget( jahwidgets.find( "footer", self.handle ), state )

		# Show the window
		self.handle.show( True )

#
# Main method
#

def main( ):
	# Turn on trace if required
	if len( sys.argv ) > 1 and sys.argv[ 1 ] == "--trace":
		sys.settrace( utilities.traceit )
		
	# Initialise the application
	app = jahwidgets.application( )

	# setup some styling
	if not app.setStyle( "v3" ):
		print "failed to load v3 theme"

	# Initialise the open libraries
	openassetlib.init( uuid_generator( ) )

	# All objects will share this state object
	state = jahstate.app_state( app )

	# Attempt to restore
	state.load_reels( )

	# Create the main jplayer instance
	jah = jahplayer( state )

	# Execute the application
	app.start_event_loop( )

	# Save state
	state.save_reels( )

	# Close opl
	openassetlib.uninit( )

#
# Run the main method if required
#

if __name__ == "__main__":
	main( )
