# jahplayer - A widget library representation.

# Copyright (C) 2006 VM Inc.
# Released under the GPL.
# For more information, see http://www.jahshaka.org.

import openpluginlib
import assets
import utilities
import os.path
import jahwidgets
import openimagelib

#
# Commmon state for the application
#

class app_state:
	# Constructor
	def __init__( self, app ):
		self.app = app

		# Current operating reel mode
		self.mode = ""
		self.mode_listeners = [ ]

		# Current preview
		self.preview = ""
		self.preview_listeners = [ ]

		# Current presentation
		self.presentation = ""
		self.presentation_listeners = [ ]

		# Collection of reels
		self.reels = [ ]
		self.reel_names = [ ]
		self.reel = None
		self.reel_listeners = [ ]

		# Active media listeners
		self.media_listeners = [ ]

		# List of functions to prepare media
		self.media_prepare = [ ]

		# 'Global' view port player stuff (needed for viewport and controller)
		self.player = None
		self.player_handle = None
		self.position_prop = None
		
		# Instantiate a sqlite3 database plugin
		self.database_plugin = None
		discoveryResults = openpluginlib.discovery( openpluginlib.all_query_traits( "openassetlib", "", "sqlite3", 0 ) )
		if discoveryResults.size() == 0:
			print "ERROR: No sqlite3 database plugin discovered!"
		else:
			for resultIt in discoveryResults:
				#print resultIt.libname(), ':', resultIt.name(), resultIt.merit()
 				plugin = resultIt.create_plugin(utilities.get_absolute_reel( "oal_sqlite3.db" ))
 				if plugin is not None:
 					self.database_plugin = plugin
 					break
 			if self.database_plugin is None:
 				print "ERROR: Failed to create sqlite3 database plugin"

	# Get the database plugin		
	def get_database_plugin( self ):
		return self.database_plugin
	
	# Register a mode notifier
	def register_media_preparation( self, object ):
		self.media_prepare.append( object )

	def prepare_media( self, media ):
		for i in self.media_prepare:
			i( media )

	# Register a mode notifier
	def register_mode_listener( self, object ):
		self.mode_listeners.append( object )

	# Notify all mode listeners of a change
	def notify_mode_listeners( self ):
		for i in self.mode_listeners:
			i.notify_mode_change( self.mode )

	# Set the active mode
	def set_active_mode( self, mode ):
		self.mode = mode
		self.notify_mode_listeners( )

	# Get the active mode
	def get_active_mode( self ):
		return self.mode

	# Register a preview listener
	def register_preview_listener( self, object ):
		self.preview_listeners.append( object )

	# Notify all preview listeners of a change
	def notify_preview_listeners( self ):
		for i in self.preview_listeners:
			i.notify_preview_change( self.preview )

	# Set the active preview
	def set_preview( self, preview ):
		self.preview = preview
		self.notify_preview_listeners( )

	# Get the active preview
	def get_preview( self ):
		return self.preview

	# Register a presentation listener
	def register_presentation_listener( self, object ):
		self.presentation_listeners.append( object )

	# Notify all presentation listeners of a change
	def notify_presentation_listeners( self ):
		for i in self.presentation_listeners:
			i.notify_presentation_change( self.presentation )

	# Set the active presentation
	def set_presentation( self, presentation ):
		self.presentation = presentation
		self.notify_presentation_listeners( )

	# Get the active presentation
	def get_presentation( self ):
		return self.presentation

	# Register a reel notifier
	def register_reel_listener( self, object ):
		self.reel_listeners.append( object )

	# Notify all reel listeners of a change
	def notify_reel_listeners( self ):
		for i in self.reel_listeners:
			i.notify_reel_change( )

	# Set the active reel
	def set_active_reel( self, reel ):
		if self.reel is not None:
			self.reel.save( self.database_plugin )
			self.reel.active = ""
		if not reel in self.reels:
			self.reels.append( reel )
			path, name = os.path.split( reel.filename )
			self.reel_names.append( name )
		self.reel = reel
		self.notify_reel_listeners( )

	# Get the active reel
	def get_active_reel( self ):
		if self.reel is None and len( self.reels ) == 0:
			r = assets.collection( )
			self.set_active_reel( r )
		elif self.reel is None:
			self.set_active_reel( self.reels[ 0 ] )
		return self.reel

	# Register a media notifier
	def register_media_listener( self, object ):
		self.media_listeners.append( object )

	# Notify all reel listeners of a change
	def notify_media_listeners( self, media ):
		for i in self.media_listeners:
			i.notify_media_change( media )

	# Get the active media in the current reel
	def get_active_media( self ):
		reel = self.get_active_reel( )
		return reel.get_active_media( )

	# Set the active media in the current reel
	def set_active_media( self, media ):
		if media is not None and media.asset is not None:
			reel = media.reel
			if not reel.is_active_media( media ):
				if reel.set_active_media( media ):
					self.notify_reel_listeners( )
					self.notify_media_listeners( media )

	# Set the active media in the current reel
	def remove_media( self, media ):
		reel = media.reel
		if reel is not None:
			reel.remove( guid = media.asset.guid( ) )
			self.notify_reel_listeners( )

	def clear_reel( self, reel = None ):
		if reel is None: reel = self.get_active_reel( )
		reel.clear( )
		self.notify_reel_listeners( )

	def is_image_extension( self, extension ):
		all_extensions = ""

		oil_plugins = openpluginlib.discovery( openpluginlib.all_query_traits( "openimagelib", "", "", 0 ) )
		for y in oil_plugins:
			all_extensions = all_extensions + y.in_filter( )

 		extension = extension.lower( )
 		return all_extensions.find( extension ) != -1

	def is_image_sequence( self, list ):
		# To simplify use, we will only do an image sequence on a single item
		if len( list ) != 1: return None

		for item in list:
			path, name = os.path.split( item )
			prefix, ext = os.path.splitext( name )

		if not self.is_image_extension( ext ) or not prefix[ len( prefix ) - 1 ].isdigit( ):
			return None

		stripped = prefix
		expr = ""
		while stripped[ len( stripped ) - 1 ].isdigit( ):
			stripped = stripped[ 0:len( stripped ) - 1 ]
			expr += "[0-9]"

		if expr == "":
			return None

		contents = utilities.get_files( path, stripped + expr + ext )

		start = end = -1

		# First parse: Determine the longest common prefix
		prefix = os.path.commonprefix( contents )

		# Second parse: Check that all files are 'prefixNNN.ext'
		for item in contents:

			path, ext = os.path.splitext( item )
			number = path[ len( prefix ):len( path ) ]

			if number == "":
				return None

			elif start == -1:
				start = end = int( number )

			elif int( number ) == end + 1:
				end = int( number )

			else:
				return None

		# Third parse: Check that all images are loadable and have a common dimension
		width = height = 0

		for item in contents:
			break
			media = jahwidgets.create_media( item + "/sequence:" )
			media.open( )
			image = media.fetch_image( )

			if image is None:
				return None

			elif width == 0 and height == 0:
				width = image.width( )
				height = image.height( )

			elif width != image.width( ) or height != image.height( ):
				return None

		while prefix[ len( prefix ) - 1 ].isdigit( ):
			prefix = prefix[ 0:len( prefix ) - 1 ]

		return prefix + expr + "\\" + ext + "/sequence:cache=" + str( end - start + 1 ) + ":regexp=1"

	# Append items on to the active reel
	def process_media( self, list ):
		reel = self.get_active_reel( )
		uri = self.is_image_sequence( list )
		if uri is not None:
			reel.append( uri )
			self.notify_reel_listeners( )
			self.app.processEvents( 0 )
		else:
			for i in list:
				reel.append( i )
				self.notify_reel_listeners( )
				self.app.processEvents( 0 )

	# Utility method for playlist oriented functionality
	def play_next( self, current = None ):
		if current == None: current = self.player.active
		reel = current.reel
		if reel is not None:
			index = reel.find( current )
			if index != len( reel.playlist ):
				index = index + 1
				if index >= len( reel.playlist ): index = 0
				self.set_active_media( reel.get_media( index ) )
			return index
		return -1

	# Utility method for playlist oriented functionality
	def play_prev( self, current = None ):
		if current == None: current = self.player.active
		reel = current.reel
		if reel is not None:
			index = reel.find( current )
			if index != len( reel.playlist ):
				index = index - 1
				if index < 0: index = len( reel.playlist ) - 1
				self.set_active_media( reel.get_media( index ) )
			return index
		return -1

	# Attempt to restore
	def load_reels( self ):
		for f in utilities.get_files( utilities.get_absolute_reel( "" ), ".*\.reel$" ):
			reel = assets.collection( )
			reel.load( self.database_plugin, f )
			self.reel = reel
			self.set_active_reel( reel )

	def save_reels( self ):
		for reel in self.reels:
			reel.save( self.database_plugin )

	def new_reel( self ):
		path = utilities.get_absolute_reel( "" )
		for i in range( 999 ):
			existing = utilities.get_files( path, "default%03d.reel" % i )
			if len( existing ) != 1:
				reel = assets.collection( )
				reel.load( self.database_plugin, utilities.get_absolute_reel( "default%03d.reel" % i ) )
				reel.save( self.database_plugin )
				self.set_active_reel( reel )
				break

	def reel_next( self ):
		if len( self.reels ) > 1:
			index = 0
			for reel in self.reels:
				if reel is self.get_active_reel( ): break
				index += 1
			if index < len( self.reels ):
				index += 1
				if index >= len( self.reels ): index = 0
				self.set_active_reel( self.reels[ index ] )

	def reel_prev( self ):
		if len( self.reels ) > 1:
			index = 0
			for reel in self.reels:
				if reel is self.get_active_reel( ): break
				index += 1
			if index < len( self.reels ):
				index -= 1
				if index < 0: index = len( self.reels ) - 1
				self.set_active_reel( self.reels[ index ] )


