# jahplayer - A widget library representation.

# Copyright (C) 2006 VM Inc.
# Released under the GPL.
# For more information, see http://www.jahshaka.org.

import jahwidgets
import jahstate
import utilities

#
# Geometry delegate
#

class geometry_delegate( jahwidgets.player_delegate ):
	# Constructor
	def __init__( self, viewport ):
		jahwidgets.player_delegate.__init__( self )
		self.viewport = viewport
		self.method = self.scale_fill
		self.zoom = 1.0
		self.x = 0.5
		self.y = 0.5
		self.mode = "preview_a"
		self.mode_method = self.preview_a
		self.showing = 1
		self.replace = 0
		self.timer = jahwidgets.timer( )
		self.next_asset = None

	def show( self, a, a_on = True, b = None, b_on = False ):
		self.viewport.player.clear( )

		self.showing = 0
		if a_on: self.showing += 1
		if b_on: self.showing += 1
		
		if a is not None:
			a.visible = a_on
			self.viewport.state.prepare_media( a )
			self.viewport.player.push_back( a )
		if b is not None:
			b.visible = b_on
			self.viewport.state.prepare_media( b )
			self.viewport.player.push_back( b )

		self.viewport.player.refresh( )

	def preview_a( self ):
		self.viewport.active_a.offset_w = 0.0
		self.viewport.active_a.usage_w = 1.0
		self.show( self.viewport.active_a )
		self.replace = 0
		self.viewport.ignore = True
		if self.viewport.state.player.active.asset is not self.viewport.active_a:
			self.viewport.state.set_active_media( self.viewport.active_a )
		self.viewport.ignore = False

	def preview_b( self ):
		self.viewport.active_b.offset_w = 0.0
		self.viewport.active_b.usage_w = 1.0
		self.show( self.viewport.active_a, False, self.viewport.active_b, True )
		self.replace = 1
		self.viewport.ignore = True
		if self.viewport.state.player.active.asset is not self.viewport.active_b:
			self.viewport.state.set_active_media( self.viewport.active_b )
		self.viewport.ignore = False

	def preview_ab( self ):
		self.viewport.active_a.offset_w = 0.0
		self.viewport.active_a.usage_w = 1.0
		self.viewport.active_b.offset_w = 0.0
		self.viewport.active_b.usage_w = 1.0
		self.show( self.viewport.active_a, True, self.viewport.active_b, True )
		self.viewport.ignore = True
		if self.viewport.player.active is self.viewport.active_a: 
			self.viewport.state.set_active_media( self.viewport.active_b )
			self.replace = 1
		else:
			self.viewport.state.set_active_media( self.viewport.active_a )
			self.replace = 0
		self.viewport.ignore = False

	def preview_ab_plus( self ):
		self.viewport.active_a.offset_w = 0.0
		self.viewport.active_a.usage_w = 0.5
		self.viewport.active_b.offset_w = 0.5
		self.viewport.active_b.usage_w = 0.5
		self.show( self.viewport.active_a, True, self.viewport.active_b, True )
		self.showing = 1
		self.viewport.player.refresh( )

	def play_next( self ):
		media = self.viewport.player.active
		if media is not None and media.reel is not None and media.asset is not None:
			asset = media.asset
			reel = media.reel
			index = reel.find( guid = asset.guid( ) )
			if index != len( reel.playlist ):
				index = index + 1
				if index >= len( reel.playlist ): index = 0
				if self.next_asset is not reel.get_media( index ).asset:
					self.next_asset = reel.get_media( index ).asset
					self.viewport.state.set_active_media( reel.get_media( index ) )

	# Determine if the media is needed
	def media_needed( self, media ):
		if media is not self.viewport.player.active and media.length > 1:
			media.refresh( self.viewport.player.active.position( ) % media.length, False )
		return True

	# Place the media item in the reel
	def media_geometry( self, media, width, height ):
		view_width = self.viewport.player.width
		view_height = self.viewport.player.height
		self.method( media, width, height, view_width, view_height )

	# Allows caching
	def media_rendered( self, media ):
		length = media.out_point
		mode = self.viewport.state.get_active_mode( )
		if mode in [ "mode_timeline", "mode_playlist" ] and self.viewport.player.speed == 1 and media is self.viewport.player.active and length > 1 and media.position( ) >= length - 1:
			self.play_next( )

	# Set geometry
	def set( self, media, width, height, view_width, view_height ):

		if self.showing == 1:
			width *= self.zoom
			height *= self.zoom

			if width < view_width: 
				self.x, x = 0.5, ( view_width - width ) / 2
			else: 
				x = view_width * self.x - width / 2
				if x < 0 and width + x < view_width: x = view_width - width
				if x > 0: x = 0

			if height < view_height: 
				self.y, y = 0.5, ( view_height - height ) / 2
			else: 
				y = view_height * self.y - height / 2
				if y < 0 and height + y < view_height: y = view_height - height
				if y > 0: y = 0

			media.set_geometry( int( x ), int( y ), int( width ), int( height ) )
		else:
			view_width /= 2
			width, height = utilities.bind_size_to( width, height, view_width, view_width / media.ar( ) )
			x = ( view_width - width ) / 2
			if media is not self.viewport.active_a: x += view_width
			media.set_geometry( x, ( view_height - height ) / 2, int( width ), int( height ) )

	# Scale to fill
	def scale_fill( self, media, width, height, view_width, view_height ):
		width, height = utilities.bind_size_to( width, height, view_width, view_height )
		self.set( media, width, height, view_width, view_height )

	# Scale to fill vertically
	def scale_vert( self, media, width, height, view_width, view_height ):
		width, height = utilities.bind_size_to( width, height, view_height * media.ar( ), view_height )
		self.set( media, width, height, view_width, view_height )

	# Scale to fill horizontally
	def scale_horiz( self, media, width, height, view_width, view_height ):
		width, height = utilities.bind_size_to( width, height, view_width, view_width / media.ar( ) )
		self.set( media, width, height, view_width, view_height )

	# No scaling - show pixel size
	def scale_off( self, media, width, height, view_width, view_height ):
		self.set( media, width, height, view_width, view_height )

#
# Drag n Drop delegate
#

class drop_delegate( jahwidgets.drop_delegate ):
	def __init__( self, viewport ):
		jahwidgets.drop_delegate.__init__( self )
		self.viewport = viewport
		self.state = viewport.state

	def accept( self ):
		return 1

	def operate( self ):
		self.state.process_media( self )

#
# Mouse delegate
#

class mouse_delegate( jahwidgets.mouse_delegate ):
	def __init__( self, viewport ):
		jahwidgets.mouse_delegate.__init__( self )
		self.viewport = viewport
		self.geometry = viewport.view_geometry
		self.state = viewport.state
		self.player = viewport.player
		self.jog_x = 0
		self.moving = None
		self.orig_x = 0.0
		self.orig_y = 0.0
		self.start_x = 0.0
		self.start_y = 0.0

	def mousePressEvent( self ):
		self.accept( )
		media = self.player.find( self.x, self.y )

		if not media:
			media = self.player.active

		if self.isRightButton:
			self.jog_speed = self.player.speed
			self.jog_x = self.x
			self.player.speed = 0
		elif self.isMiddleButton:
			self.player.speed = not self.player.speed
		elif self.isLeftButton and self.isDoubleClick:
			if media is not self.player.active:
				self.player.active = media
				self.geometry.replace = 0
				if media is self.viewport.active_a: self.geometry.replace = 1
				self.viewport.ignore = True
				self.state.set_active_media( media )
				self.viewport.ignore = False
		elif self.isLeftButton and media:
			self.moving = media
			self.orig_x = self.geometry.x
			self.orig_y = self.geometry.y
			self.start_x = self.x 
			self.start_y = self.y

	def mouseMoveEvent( self ):
		self.accept( )
		if self.jog_x > 0:
			self.player.speed = 2 * ( ( self.x - self.jog_x ) / 20 )
		elif self.moving is not None:
			self.geometry.x = self.orig_x + float( self.x - self.start_x ) / self.player.width 
			self.geometry.y = self.orig_y + float( self.y - self.start_y ) / self.player.height
			self.player.refresh( )

	def mouseReleaseEvent( self ):
		self.accept( )
		if self.jog_x:
			self.player.speed = self.jog_speed
			self.jog_x = 0
		elif self.moving is not None:
			self.moving = None

#
# Wheel delegate
#

class wheel_delegate( jahwidgets.wheel_delegate ):
	def __init__( self, viewport ):
		jahwidgets.wheel_delegate.__init__( self )
		self.geometry = viewport.view_geometry
		self.state = viewport.state
		self.player = viewport.player

	def wheelEvent( self ):
		self.accept( )

		offset = self.delta / 12.0
		diff = offset * 0.002

		if diff > 0 or ( diff < 0 and self.geometry.zoom > 0.01 ):
			self.geometry.zoom += diff
			self.player.refresh( )

#
# The viewport player class
#

class widget:
	# Constuctor for the player
	def __init__( self, parent, state ):
		# Construct the ui
		self.handle = jahwidgets.create_ui_from_file( utilities.get_absolute_path( "viewport.ui" ), parent )

		# For multiple modes, we need to maintain two active clips
		self.active_a = jahwidgets.create_media( utilities.get_absolute_path( "jahplayer-logo.jpg" ) )
		self.active_a.zoom_active = True
		self.active_a.reel = None
		self.active_b = jahwidgets.create_media( utilities.get_absolute_path( "jahplayer-logo.jpg" ) )
		self.active_b.zoom_active = True
		self.active_b.reel = None

		# Store state and register for reel changes
		self.state = state
		self.state.register_media_listener( self )
		self.state.register_preview_listener( self )
		self.state.register_presentation_listener( self )
		self.state.register_reel_listener( self )

		# Ignore a notification of a media change
		self.ignore = False

		# Set up the delegated player for this widget
		self.player_handle = jahwidgets.find( "player", self.handle )
		self.player = jahwidgets.player( self.player_handle )

		# Register the player handles with the state to allow the controller access
		self.state.player_handle = self.player_handle
		self.state.player = self.player
		self.state.change_zoom = self.change_zoom

		# Construct delegates
		self.view_geometry = geometry_delegate( self )
		self.view_dnd = drop_delegate( self )
		self.view_mouse = mouse_delegate( self )
		self.view_wheel = wheel_delegate( self )

		# Register them
		self.player.register_player_delegate( self.view_geometry )
		self.player.register_drop_delegate( self.view_dnd )
		self.player.register_mouse_delegate( self.view_mouse )
		self.player.register_wheel_delegate( self.view_wheel )

		# Set up the default behaviour
		self.notify_media_change( None )

	def replace( self, media, previous ):
		result = None
		if media is not None:
			if previous is None or previous.asset is not media.asset:
				active = self.player.active is previous
				result = media.reel.get_sequence( media.asset )
				if result == None:
					result = jahwidgets.create_media( str( media.asset.find( "file" ).second ) )
				sequence = str( media.asset.find( "sequence" ).first ) != ""
				if result is not None and result.open( ):
					result.reel = media.reel
					result.asset = media.asset
					result.zoom_active = True
					#result.reel.set_sequence( result )
					if active:
						self.player.show_all_frames = sequence or result.length <= 1
						speed = 1
						if result.length <= 1:
							speed = 0
						self.state.prepare_media( result )
						self.player.play( result, speed )
				else:
					return previous
			else:
				result = previous
		return result

	# Handler for reel changes
	def notify_media_change( self, media ):
		if not self.ignore and media is not None:
			if self.view_geometry.replace == 0:
				self.active_a = self.replace( media, self.active_a )
			else: 
				self.active_b = self.replace( media, self.active_b )
			self.view_geometry.mode_method( )
		elif not self.ignore:
			self.player.play( self.active_a )
			self.player.speed = 0

	def notify_reel_change( self ):
		reel = self.active_a.reel
		if self.active_a.asset is not None and reel is not None and reel.get_media( guid = self.active_a.asset.guid( ) ) is None:
			active = self.player.active is self.active_a
			self.active_a = jahwidgets.create_media( utilities.get_absolute_path( "jahplayer-logo.jpg" ) )
			self.active_a.reel = None
			self.active_a.zoom_active = True
			if active: self.player.active = self.active_a
		reel = self.active_b.reel
		if self.active_b.asset is not None and reel is not None and reel.get_media( guid = self.active_b.asset.guid( ) ) is None:
			active = self.player.active is self.active_b
			self.active_b = jahwidgets.create_media( utilities.get_absolute_path( "jahplayer-logo.jpg" ) )
			self.active_b.reel = None
			self.active_b.zoom_active = True
			if active: self.player.active = self.active_b
		if self.player.active.length == 1: self.player.speed = 0
		self.view_geometry.mode_method( )
		self.player.refresh( )

	# Handler for preview change
	def notify_preview_change( self, preview ):
		self.view_geometry.mode = preview
		self.view_geometry.mode_method = getattr( self.view_geometry, preview )
		self.view_geometry.mode_method( )

	# Handler for presentation change
	def notify_presentation_change( self, presentation ):
		self.view_geometry.method = getattr( self.view_geometry, presentation )
		self.view_geometry.x = 0.5
		self.view_geometry.y = 0.5
		self.view_geometry.zoom = 1.0
		self.player.refresh( )

	def change_zoom( self, diff ):
		if diff > 0 or ( diff < 0 and self.view_geometry.zoom > 0.01 + diff ):
			self.view_geometry.zoom += diff
			self.player.refresh( )


